"""HOS-M2F: Markdown to Industry Standard Format Compiler Engine"""

__version__ = "1.0.0"
__author__ = "HOS Team"

# 尝试导入模块，但如果失败也不会中断
Engine = None
try:
    from hos_m2f.engine.engine import Engine
    from hos_m2f.pipeline import *
    __all__ = ["Engine", *pipeline.__all__]
except ImportError as e:
    # 如果导入失败，只导出版本信息
    __all__ = []